@@ -35,11 +35,17 @@ static const char* ifaceName = "org.lxde.LxImage.Application";
 
 Application::Application(int& argc, char** argv):
   QApplication(argc, argv),
-  windowCount_(0),
-  libFm() {
+  libFm(),
+  windowCount_(0) {
+  setApplicationVersion(LXIMAGE_VERSION);
 }
 
 bool Application::init(int argc, char** argv) {
+  Q_UNUSED(argc)
+  Q_UNUSED(argv)
+
+  setAttribute(Qt::AA_UseHighDpiPixmaps, true);
+
   // install the translations built-into Qt itself
   qtTranslator.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
   installTranslator(&qtTranslator);
@@ -62,7 +68,7 @@ bool Application::init(int argc, char** argv) {
     new ApplicationAdaptor(this);
     dbus.registerObject("/Application", this);
 
-    connect(this, SIGNAL(aboutToQuit()), SLOT(onAboutToQuit()));
+    connect(this, &Application::aboutToQuit, this, &Application::onAboutToQuit);
 
     if(settings_.useFallbackIconTheme())
       QIcon::setThemeName(settings_.fallbackIconTheme());
@@ -81,6 +87,7 @@ bool Application::init(int argc, char** argv) {
 bool Application::parseCommandLineArgs() {
   QCommandLineParser parser;
   parser.addHelpOption();
+  parser.addVersionOption();
 
   QCommandLineOption screenshotOption(
     QStringList() << "s" << "screenshot",
@@ -152,7 +159,9 @@ void Application::newWindow(QStringList files) {
       if(settings_.windowMaximized())
         window->setWindowState(window->windowState() | Qt::WindowMaximized);
 
-      window->show();
+      /* when there's an image, we show the window AFTER resizing
+         and centering it appropriately at MainWindow::updateUI() */
+      //window->show();
     }
   }
 }
@@ -175,6 +184,5 @@ void Application::editPreferences() {
 }
 
 void Application::onAboutToQuit() {
-  qDebug("aboutToQuit");
   settings_.save();
 }
